<?php

namespace App\Controller;

use App\Entity\Fault;
use App\Form\FaultType;
use App\Repository\FaultRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;

#[Route('/fault')]
final class FaultController extends AbstractController
{
    #[Route(name: 'app_fault_index', methods: ['GET'])]
    public function index(FaultRepository $faultRepository): Response
    {
        // Filtrage des pannes : un chauffagiste ne peut voir que ses pannes
        $faults = $this->isGranted('ROLE_CHAUFFAGISTE')
            ? $faultRepository->findByUser($this->getUser())  // Filtre les pannes par utilisateur
            : $faultRepository->findAll(); // Admins voient toutes les pannes

        return $this->render('fault/index.html.twig', [
            'faults' => $faults,
        ]);
    }

    #[Route('/new', name: 'app_fault_new', methods: ['GET', 'POST'])]
    public function new(Request $request, EntityManagerInterface $entityManager): Response
    {
        $fault = new Fault();
        $form = $this->createForm(FaultType::class, $fault);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            // Associe la panne à un chauffagiste si c'est un chauffagiste
            if ($this->isGranted('ROLE_CHAUFFAGISTE')) {
                $fault->setUser($this->getUser());
            }

            $entityManager->persist($fault);
            $entityManager->flush();

            return $this->redirectToRoute('app_fault_index', [], Response::HTTP_SEE_OTHER);
        }

        return $this->render('fault/new.html.twig', [
            'fault' => $fault,
            'form' => $form,
        ]);
    }

    #[Route('/{id}', name: 'app_fault_show', methods: ['GET'])]
    public function show(Fault $fault): Response
    {
        // Un chauffagiste ne peut voir que ses pannes
        if ($this->isGranted('ROLE_CHAUFFAGISTE') && $fault->getUser() !== $this->getUser()) {
            throw $this->createAccessDeniedException('Vous ne pouvez pas voir cette panne.');
        }

        return $this->render('fault/show.html.twig', [
            'fault' => $fault,
        ]);
    }

    #[Route('/{id}/edit', name: 'app_fault_edit', methods: ['GET', 'POST'])]
    public function edit(Request $request, Fault $fault, EntityManagerInterface $entityManager): Response
    {
        // Un chauffagiste ne peut modifier que ses propres pannes
        if ($this->isGranted('ROLE_CHAUFFAGISTE') && $fault->getUser() !== $this->getUser()) {
            throw $this->createAccessDeniedException('Vous ne pouvez pas modifier cette panne.');
        }

        $form = $this->createForm(FaultType::class, $fault);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager->flush();

            return $this->redirectToRoute('app_fault_index', [], Response::HTTP_SEE_OTHER);
        }

        return $this->render('fault/edit.html.twig', [
            'fault' => $fault,
            'form' => $form,
        ]);
    }

    #[Route('/{id}', name: 'app_fault_delete', methods: ['POST'])]
    public function delete(Request $request, Fault $fault, EntityManagerInterface $entityManager): Response
    {
        // Un chauffagiste ne peut supprimer que ses propres pannes
        if ($this->isGranted('ROLE_CHAUFFAGISTE') && $fault->getUser() !== $this->getUser()) {
            throw $this->createAccessDeniedException('Vous ne pouvez pas supprimer cette panne.');
        }

        if ($this->isCsrfTokenValid('delete'.$fault->getId(), $request->get('csrf_token'))) {
            $entityManager->remove($fault);
            $entityManager->flush();
        }

        return $this->redirectToRoute('app_fault_index', [], Response::HTTP_SEE_OTHER);
    }
}
